/*
----------------------------------------------------------------------

	Clock art

	Alain Chebrou

	clock.h		The clock art code

	When		Who	What
	01/02/21	ac	Creation

----------------------------------------------------------------------
*/

#if (! defined CLOCK_H_)
#define CLOCK_H_
//--------------------------------------------------------------------------------

#define		PROTO_BOARD			0			// 0 for real board, 1 for proto board

#define		MOTOR_COUNT			8u			// Count of managed hands
#define		MOTOR_TIMER_ID		7u
#define		MOTOR_TIMER_FREQ	2000u
#define		MOTOR_TIMER_PRIO	BSP_MAX_INT_PRIO

#define		SPEED_HIGHEST		2u			// Ticks per pulse

#define		PULSE_PER_DEG		12u
#define		PULSE_PER_TURN		(360u * PULSE_PER_DEG)

#define		DIR_CCW				0u
#define		DIR_CW				1u
#define		DIR_SHORT			2u		// Use the shortest way to go to the requested angle
#define		DIR_LONG			3u		// Use the longest way to go to the requested angle

//--------------------------------------------------------------------------------
//	Define resources
//	The 2 LEDS and the button are defined in bspboard.h

#if (PROTO_BOARD == 0)

// For real board

#define	PORT_TST0		'B'		// Test pin wired as TP1
#define	PIN_TST0		14

#define	PORT_TST1		'B'		// Test pins not wired
#define	PIN_TST1		10


#define	PORT_CMDACK		'B'		// Command ACK
#define	PIN_CMDACK		12

#define	PORT_MRST		'C'		// Motor RESET
#define	PIN_MRST		6

#define	PORT_MDIR		'C'		// Motor direction
#define	PIN_MDIR		7

#define	PORT_M0			'B'		// Motors frequency
#define	PIN_M0			6

#define	PORT_M1			'B'
#define	PIN_M1			5

#define	PORT_M2			'B'
#define	PIN_M2			3

#define	PORT_M3			'B'
#define	PIN_M3			4

#define	PORT_M4			'D'
#define	PIN_M4			0

#define	PORT_M5			'D'
#define	PIN_M5			3

#define	PORT_M6			'D'
#define	PIN_M6			1

#define	PORT_M7			'D'
#define	PIN_M7			2


#define	PORT_ADDR0		'C'		// CPU address
#define	PIN_ADDR0		14

#define	PORT_ADDR1		'C'
#define	PIN_ADDR1		13

#define	PORT_ADDR2		'B'
#define	PIN_ADDR2		8

#define	PORT_ADDR3		'B'
#define	PIN_ADDR3		9


#define	PORT_HL0		'A'		// Hall sensors
#define	PIN_HL0			0		// Motor 0, long hand

#define	PORT_HS0		'A'
#define	PIN_HS0			4		// Motor 0, short hand

#define	PORT_HL1		'B'
#define	PIN_HL1			1		// Motor 1, long hand

#define	PORT_HS1		'B'
#define	PIN_HS1			2		// Motor 1, short hand

#define	PORT_HL2		'A'
#define	PIN_HL2			7		// Motor 2, long hand

#define	PORT_HS2		'B'
#define	PIN_HS2			0		// Motor 2, short hand

#define	PORT_HL3		'A'
#define	PIN_HL3			5		// Motor 3, long hand

#define	PORT_HS3		'A'
#define	PIN_HS3			6		// Motor 3, short hand


#define	PORT_MOSI		'A'		// Command SPI
#define	PIN_MOSI		12

#define	PORT_MISO		'A'		// Not used
#define	PIN_MISO		11

#define	PORT_SCK		'A'
#define	PIN_SCK			1

#define	PORT_NSS		'A'
#define	PIN_NSS			15


#define	PORT_CMDTX		'A'		// Command UART
#define	PIN_CMDTX		9

#define	PORT_CMDRX		'A'
#define	PIN_CMDRX		10

#else	// PROTO_BOARD

// For proto board
#define	PORT_TST0		'B'		// Test pins
#define	PIN_TST0		13

#define	PORT_TST1		'B'
#define	PIN_TST1		14


#define	PORT_CMDACK		'B'		// Command ACK
#define	PIN_CMDACK		12

#define	PORT_MRST		'C'		// Motor RESET
#define	PIN_MRST		6

#define	PORT_MDIR		'C'		// Motor direction
#define	PIN_MDIR		7

#define	PORT_M0			'D'		// Motors frequency
#define	PIN_M0			0

#define	PORT_M1			'D'
#define	PIN_M1			1

#define	PORT_M2			'D'
#define	PIN_M2			2

#define	PORT_M3			'D'
#define	PIN_M3			3

#define	PORT_M4			'B'
#define	PIN_M4			3

#define	PORT_M5			'B'
#define	PIN_M5			4

#define	PORT_M6			'B'
#define	PIN_M6			5

#define	PORT_M7			'B'
#define	PIN_M7			6


#define	PORT_ADDR0		'B'		// CPU address
#define	PIN_ADDR0		8

#define	PORT_ADDR1		'B'
#define	PIN_ADDR1		9

#define	PORT_ADDR2		'C'
#define	PIN_ADDR2		13

#define	PORT_ADDR3		'C'
#define	PIN_ADDR3		14


#define	PORT_HL0		'A'		// Hall sensors
#define	PIN_HL0			0		// Motor 0, long hand

#define	PORT_HS0		'A'
#define	PIN_HS0			4		// Motor 0, short hand

#define	PORT_HL1		'A'
#define	PIN_HL1			5		// Motor 1, long hand

#define	PORT_HS1		'A'
#define	PIN_HS1			6		// Motor 1, short hand

#define	PORT_HL2		'A'
#define	PIN_HL2			7		// Motor 2, long hand

#define	PORT_HS2		'B'
#define	PIN_HS2			0		// Motor 2, short hand

#define	PORT_HL3		'B'
#define	PIN_HL3			1		// Motor 3, long hand

#define	PORT_HS3		'B'
#define	PIN_HS3			2		// Motor 3, short hand


#define	PORT_MOSI		'A'		// Command SPI
#define	PIN_MOSI		12

#define	PORT_MISO		'A'
#define	PIN_MISO		11

#define	PORT_SCK		'A'
#define	PIN_SCK			1

#define	PORT_NSS		'A'
#define	PIN_NSS			15


#define	PORT_CMDTX		'A'		// Command UART
#define	PIN_CMDTX		9

#define	PORT_CMDRX		'A'
#define	PIN_CMDRX		10

#endif	// PROTO_BOARD

//--------------------------------------------------------------------------------
// The motor RESET is active low: motorResetOn() sets the pin low
__ALWAYS_STATIC_INLINE	void	motorResetOn (void)
{
	BSP_IO_GPIO(PORT_MRST - 'A')->BRR = (1 << PIN_MRST) ;
}

__ALWAYS_STATIC_INLINE	void	motorResetOff (void)
{
	BSP_IO_GPIO(PORT_MRST - 'A')->BSRR = (1 << PIN_MRST) ;
}

__ALWAYS_STATIC_INLINE	void	motorDirCCW (void)
{
	BSP_IO_GPIO(PORT_MDIR - 'A')->BRR = (1 << PIN_MDIR) ;
}

__ALWAYS_STATIC_INLINE	void	motorDirCW (void)
{
	BSP_IO_GPIO(PORT_MDIR - 'A')->BSRR = (1 << PIN_MDIR) ;
}

// Set the Cmd Ack pin level high (Open Drain)
__ALWAYS_STATIC_INLINE	void	cmdAckOn (void)
{
	BSP_IO_GPIO(PORT_CMDACK - 'A')->BSRR = (1 << PIN_CMDACK) ;
}

// Set the Cmd Ack pin level low
__ALWAYS_STATIC_INLINE	void	cmdAckOff (void)
{
	BSP_IO_GPIO(PORT_CMDACK - 'A')->BRR = (1 << PIN_CMDACK) ;
}

// Set test points
__ALWAYS_STATIC_INLINE	void	tst0_0 (void)
{
	BSP_IO_GPIO(PORT_TST0 - 'A')->BRR = (1 << PIN_TST0) ;
}

__ALWAYS_STATIC_INLINE	void	tst0_1 (void)
{
	BSP_IO_GPIO(PORT_TST0 - 'A')->BSRR = (1 << PIN_TST0) ;
}

__ALWAYS_STATIC_INLINE	void	tst1_0 (void)
{
	BSP_IO_GPIO(PORT_TST1 - 'A')->BRR = (1 << PIN_TST1) ;
}

__ALWAYS_STATIC_INLINE	void	tst1_1 (void)
{
	BSP_IO_GPIO(PORT_TST1 - 'A')->BSRR = (1 << PIN_TST1) ;
}

// Set LEDs
__ALWAYS_STATIC_INLINE	void	led0_Off (void)
{
	BSP_IO_GPIO(BSP_LED0_PORT_NUMBER)->BRR = (1 << BSP_LED0_PIN_NUMBER) ;
}

__ALWAYS_STATIC_INLINE	void	led0_On (void)
{
	BSP_IO_GPIO(BSP_LED0_PORT_NUMBER)->BSRR = (1 << BSP_LED0_PIN_NUMBER) ;
}

__ALWAYS_STATIC_INLINE	void	led1_Off (void)
{
	BSP_IO_GPIO(BSP_LED1_PORT_NUMBER)->BRR = (1 << BSP_LED1_PIN_NUMBER) ;
}

__ALWAYS_STATIC_INLINE	void	led1_On (void)
{
	BSP_IO_GPIO(BSP_LED1_PORT_NUMBER)->BSRR = (1 << BSP_LED1_PIN_NUMBER) ;
}

//--------------------------------------------------------------------------------
//	in clock.c

extern	uint32_t		mcuAddress ;	// From hardware straps

void		goToOrigin		(void) ;
void		motorAckReset	(void) ;
void		spiAckSet		(void) ;

void		addDelay		(uint32_t iMotor, uint32_t msCount) ;
void		addMove			(uint32_t iMotor, uint32_t dir, uint32_t pulseCount, uint32_t msCount) ;
void		addMoveAbs		(uint32_t iMotor, uint32_t dir, uint32_t position, uint32_t msCount) ;
void		addMotorAck		(uint32_t iMotor) ;

//	Relative move: Move 'angle' degree in the 'dir' direction
//	The duration of the move is 'msCount' milli seconds

BSP_ATTR_ALLWAYS_INLINE	void	addMoveAngle (uint32_t iMotor, uint32_t dir, uint32_t angle, uint32_t msCount)
{
	addMove (iMotor, dir, angle * PULSE_PER_DEG, msCount) ;
}

//--------------------------------------------------------------------------------
//	In adc.c

void		adcInit			(void) ;
void		adcDmaInit		(uint32_t bufferSize, uint16_t * pBuffer) ;
void		adcStart		(void) ;
void		adcTrig			(void) ;

//--------------------------------------------------------------------------------
//	In packet.c

void		spiInit			(void) ;
void		spiReceive		(void) ;
void		spiCheck		(void) ;
uint32_t	isSpiReady		(void) ;

//--------------------------------------------------------------------------------
#endif /* CLOCK_H_ */
