/*
----------------------------------------------------------------------

	Clock art

	Alain Chebrou

	packet.h	The clock art code: Message packet descriptor

	When		Who	What
	10/05/21	ac	Creation

	This file is common to master board application and clock board application
	The consistency of this file between the 2 applications must be maintained

---------------------------------------------------------------------
*/

#if (! defined PACKET_H_)
#define PACKET_H_
//--------------------------------------------------------------------------------
// Packet / message structure

#define	PKT_SYNCHRO		0xEAD2u

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wpedantic"		// Avoid warning: forbids zero-size array

typedef struct
{
	uint16_t	synchro ;	// PKT_SYNCHRO
	uint16_t	size ;		// Count of 16 bits words in the packet (hdr+messages+cks)

} pktHdr_t ;

typedef struct
{
	uint8_t		size ;		// Count of 16 bits words of this command
	uint8_t		board :4 ;
	uint8_t		motor:4 ;	// Motor 0.. 7, all motors of this board if motor is 8
	uint16_t	cmd ;		// 1st command word

} msgHdr_t ;

typedef struct
{
	msgHdr_t	msgHdr ;
	uint16_t	data [0] ;

} msg_t ;

typedef struct
{
	pktHdr_t	pktHdr ;
	msg_t		Message [0] ;

} pkt_t;

#pragma GCC diagnostic pop

#define	PACKET_SIZE	1024						// Count of 16 bits words

//--------------------------------------------------------------------------------
// The command codes

#define	MSG_LONGBIT			0x8000u		// Bit for long command
#define	MSG_LONG_M			0xF800		// Mask for long command
#define	MSG_SHORT_M			0xE000		// Mask for short command

// Short command 3 bits, the higher bit is 0 (up to 4 commands, argument up to 13 bits)
#define	MSG_DELAY			0x0000u		// Delay in ms, 1 to 8191
#define	MSG_MOVE_PULSE_REL	0x2000u		// Move a count of pulse
#define	MSG_S2				0x4000u		// Unused
#define	MSG_S3				0x6000u		// Unused

// Flags for short move commands
#define	FLAG_MOVE_PLS_M		0x07FFu		// Mask for pulse value
#define	FLAG_MOVE_PLS_DIR_M	0x1800u		// Mask for direction flags
#define	FLAG_MOVE_PLS_CW	0x0000u
#define	FLAG_MOVE_PLS_CCW	0x0800u
#define	FLAG_MOVE_PLS_SHORT	0x1000u
#define	FLAG_MOVE_PLS_LONG	0x1800u


// Long command 5 bits, the higher bit is 1 (up to 16 commands, argument up to 11 bits)
#define	MSG_ZERO			(MSG_LONGBIT | 0x0000u)		// long commands
#define	MSG_MOVE_ABS		(MSG_LONGBIT | 0x0800u)		// Move a count of degree
#define	MSG_MOVE_REL		(MSG_LONGBIT | 0x1000u)		// Move a count of degree
#define	MSG_LED				(MSG_LONGBIT | 0x1800u)
#define	MSG_ACK				(MSG_LONGBIT | 0x2000u)
#define	MSG_RESET_ACK		(MSG_LONGBIT | 0x7800u)		// Command for the MCU

#define	MSG_MOTOR_ALL		8u			// Motor number for "all motors of the board"

// Flags for long move commands
#define	FLAG_MOVE_ANGLE_M	0x01FFu		// Mask for angle value
#define	FLAG_MOVE_DIR_M		0x0600u		// Mask for direction flags
#define	FLAG_MOVE_CW		0x0000u
#define	FLAG_MOVE_CCW		0x0200u
#define	FLAG_MOVE_SHORT		0x0400u
#define	FLAG_MOVE_LONG		0x0600u

//--------------------------------------------------------------------------------

#if (defined CLOCK_MASTER)

void		addMsgDelay			(uint32_t board, uint32_t motor, uint32_t delay) ;
void		addMsgMovePulseRel	(uint32_t board, uint32_t motor, uint32_t pulse, uint32_t time, uint32_t flags) ;

void		addMsgMoveAbs		(uint32_t board, uint32_t motor, uint32_t angle, uint32_t time, uint32_t flags) ;
void		addMsgMoveRel		(uint32_t board, uint32_t motor, uint32_t angle, uint32_t time, uint32_t flags) ;
void		addMsg				(uint32_t board, uint32_t motor, uint32_t command, uint32_t arg) ;
void		addResetAckAllBoards (void) ;
void		addSetAckAllBoards	(uint32_t delay) ;


void		spiInit				(void) ;
void		spiSend				(void) ;
void		packetDump			(void) ;

#endif
//--------------------------------------------------------------------------------
#endif 	// PACKET_H_
