/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	SystemClock_Config.c		The function to initialize MPU clocks
								Example for:
									STM32G071
								 These functions are examples and should be changed as needed
	When		Who	What
	18/01/21	ac	Created

----------------------------------------------------------------------
*/

#include	"aa.h"		// For BSP_DEBUG_BKPT

#include	"stm32g0xx_ll_rcc.h"
#include	"stm32g0xx_ll_system.h"
#include	"stm32g0xx_ll_pwr.h"
#include	"stm32g0xx_ll_bus.h"
#include	"stm32g0xx_ll_utils.h"

//static	void   Error_Handler	(void) ;

// ----------------------------------------------------------------------------
//	SystemClock_Config() code was generated by STM32CubeMX
//	This function must be adapted: configure the clock in accordance with your needs

//	The function must:
//		Not initialize SysTick
//		Initialize peripheral clocks
//		Set TIMPRE if requested

// ----------------------------------------------------------------------------
#if (defined STM32G0)
// ----------------------------------------------------------------------------

//	Use 8 MHz HSE as system clock source, system clock is 64 MHz
/*
void SystemClock_Config (void)
{
	LL_FLASH_SetLatency(LL_FLASH_LATENCY_2);
	while(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_2)
	{
	}

	// HSE configuration and activation
	LL_RCC_HSE_Enable();
	while(LL_RCC_HSE_IsReady() != 1)
	{
	}

	// Main PLL configuration and activation
	LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSE, LL_RCC_PLLM_DIV_1, 16, LL_RCC_PLLR_DIV_2);
	LL_RCC_PLL_Enable();
	LL_RCC_PLL_EnableDomain_SYS();
	while(LL_RCC_PLL_IsReady() != 1)
	{
	}

	// Set AHB prescaler
	LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);

	// Sysclk activation on the main PLL
	LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);
	while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
	{
	}

	// Set APB1 prescaler
	LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_1);

	LL_RCC_SetUSARTClockSource(LL_RCC_USART1_CLKSOURCE_PCLK1);
#if (defined RCC_CCIPR_USART2SEL)
	LL_RCC_SetUSARTClockSource(LL_RCC_USART2_CLKSOURCE_PCLK1);
#endif
	LL_RCC_SetADCClockSource(LL_RCC_ADC_CLKSOURCE_SYSCLK);
}
*/

// Use HSI as system clock source, system clock is 64 MHz

void SystemClock_Config(void)
{
	LL_FLASH_SetLatency(LL_FLASH_LATENCY_2);
	while(LL_FLASH_GetLatency() != LL_FLASH_LATENCY_2)
	{
	}

	// HSI configuration and activation
	LL_RCC_HSI_Enable();
	while(LL_RCC_HSI_IsReady() != 1)
	{
	}

	// Main PLL configuration and activation
	LL_RCC_PLL_ConfigDomain_SYS(LL_RCC_PLLSOURCE_HSI, LL_RCC_PLLM_DIV_1, 8, LL_RCC_PLLR_DIV_2);
	LL_RCC_PLL_Enable();
	LL_RCC_PLL_EnableDomain_SYS();
	while(LL_RCC_PLL_IsReady() != 1)
	{
	}

	// Set AHB prescaler
	LL_RCC_SetAHBPrescaler(LL_RCC_SYSCLK_DIV_1);

	// Sysclk activation on the main PLL
	LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_PLL);
	while(LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL)
	{
	}

	// Set APB1 prescaler
	LL_RCC_SetAPB1Prescaler(LL_RCC_APB1_DIV_1);

	// Peripheral clocks
	LL_RCC_SetUSARTClockSource(LL_RCC_USART1_CLKSOURCE_PCLK1);
	LL_RCC_SetUSARTClockSource(LL_RCC_USART2_CLKSOURCE_PCLK1);
	LL_RCC_SetADCClockSource(LL_RCC_ADC_CLKSOURCE_SYSCLK);
}

// ----------------------------------------------------------------------------
//	The purpose of this function is to provide a prologue and an epilogue to SystemClock_Config().

void	bspSystemClockInit ()
{
	// These initialization may be required if SystemClock_Config() configure LSE, or other
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_SYSCFG);
	LL_APB1_GRP1_EnableClock (LL_APB1_GRP1_PERIPH_PWR);

	SystemClock_Config () ;

	// Stop SysTick if it is started by SystemClock_Config()
	SysTick->CTRL = 0 ;
}

#endif

// ----------------------------------------------------------------------------
//	This function may be used by CubeMX generated code
/*
static	void   Error_Handler	(void)
{
	BSP_DEBUG_BKPT () ;
	while (1)
	{
	}
}
*/
// ----------------------------------------------------------------------------
