/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	spi25xx.h	Driver for SPI EEPROM 25AA128/25LC128
									  25AA256/25LC256
									  25AA512/25LC512

	When		Who	What
	10/05/19	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2021 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined SPI25XX128_H_
#define SPI25XX128_H_
//----------------------------------------------------------------------
#include	"spibasic.h"

typedef	void (* p25xxSlaveSelect) (spiHandle_t spiHdl, uint32_t bSelect) ;

typedef struct
{
	const spiDesc_t		* pSpiDesc ;	// Set by the user: SPI descriptor tu use
	p25xxSlaveSelect	pSlaveSelect ;	// Set by the user: Slave select callback
	spiHandle_t			spiHdl ;		// Set by the library
	uint32_t			pageSize ;		// Set by the library
	uint32_t			addrMask ;		// Set by the library

} spi25xxDesc_t ;

typedef enum
{
	SPI25XX128	= 1,	// 25xx128, 16 k bytes
	SPI25XX256	= 2,	// 25xx256, 32 k bytes
	SPI25XX512	= 3		// 25xx512, 64 k bytes

} spi25xxSize_t ;


#ifdef __cplusplus
extern "C" {
#endif

spiStatus_t	spi25xxOpen		(spi25xxDesc_t * p25xxDesc, spi25xxSize_t size, uint32_t speed) ;
void		spi25xxClose	(spi25xxDesc_t * p25xxDesc) ;
spiStatus_t	spi25xxWrite	(spi25xxDesc_t * p25xxDesc, uint32_t address, void * pBuffer, uint32_t size) ;
spiStatus_t	spi25xxRead		(spi25xxDesc_t * p25xxDesc, uint32_t address, void * pBuffer, uint32_t size) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// SPI25XX128_H_
