/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	spiG0xx.c		STM32G071xx descriptors for spibasic.c

	When		Who	What
	13/04/21	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2021 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined spiG0xx_H_
#define spiG0xx_H_
//------------------------------------------------------------------------------

/*
2 SPI: STM32G070xx STM32G071xx STM32G081xx
(3 SPI: STM32G0C1xx)

		MOSI		MISO		SCK			NSS

SPI1	PA2-0		PA6-0		PA1-0		PA4-0
		PA7_0		PA11-0		PA5-0		PA15-0
		PA12-0		PB4-0		PB3-0		PB0-0
		PB5-0		PD5-1		PD8-1		PD9-1
		PD6-1

SPI2	PA10-0		PA3-0		PA0-0		PA8-1
		PA4-1		PA9-4		PB8-1		PB9-5
		PB7-1		PB2-1		PB13-0		PB12-0
		PB11-0		PB6-4		PB10-5		PD0-1
		PB15-0		PB14-0		PD1-1
		PC3-1		PC2-1
		PD4-1		PD3-1
*/

/*
	Mode	CPOL	CPHA
	0		0		0
	1		0		1
	2		1		0
	3		1		1

	CPOL	Clock polarity controls the steady state value of the clock when no data is being transferred
			0: Low level when idle state

	CPHA	Controls which edge of the clock locks the data.
			0:first edge, 1:second edge
*/

// If half duplex communication is used, the MISO pin descriptor may be disabled (port field set to 0)

// NSS can be AA_GPIO_MODE_OUTPUT (with AF 0), if the NSS is managed by software (flag SPI_NSS_USER)
// NSS should be AA_GPIO_MODE_ALTERNATE if the auto NNS is selected (flag SPI_NSS_AUTO)

static	const spiDesc_t	spiDescArray [] =
{
	#if (WITH_SPI_1 == 1)
	{
		10,								// id
		SPI1,							// Pointer to SPI registers
		{	'A',	12u,	AA_GPIO_AF_0,	AA_GPIO_MODE_ALT_PP    | AA_GPIO_SPEED_HIGH   },		// MOSI
		{	'A',	11u,	AA_GPIO_AF_0,	AA_GPIO_MODE_ALT_PP    | AA_GPIO_SPEED_HIGH   },		// MISO
		{	'A',	1u,		AA_GPIO_AF_0,	AA_GPIO_MODE_ALT_PP_DN | AA_GPIO_SPEED_HIGH   },		// SCK
//		{	'A',	15u,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP_UP | AA_GPIO_SPEED_MEDIUM },		// NSS  AUTO/HARD
		{	'A',	15u,	AA_GPIO_AF_0,	AA_GPIO_MODE_INPUT_UP  | AA_GPIO_SPEED_LOW    },		// NSS  USER/SOFT
		{	// DMA Tx
			DMA_DMA1,							//	DMA number 1 or 2, 0 => no DMA
			1u,									//	DMA stream number 0..6
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction: LL_DMA_DIRECTION_MEMORY_TO_PERIPH
			(BSP_MAX_INT_PRIO+2),				//	DMA Interrupt priority
		},
		{	// DMA Rx
			DMA_DMA1,							//	DMA number 1 or 2, 0 => no DMA
			2u,									//	DMA stream number 0..6
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY,	//	Direction: LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			(BSP_MAX_INT_PRIO+2u),				//	DMA interrupt priority
		},
	},
	#endif	// WITH_SPI_1

	#if (WITH_SPI_2 == 1)
	{
		20,
		SPI2,
		{	'B',	11u,	AA_GPIO_AF_0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH   },		// MOSI
		{	'B',	2u,		AA_GPIO_AF_1,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH   },		// MISO
		{	'B',	8u,		AA_GPIO_AF_1,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH   },		// SCK
//		{	'B',	9u	,	AA_GPIO_AF_0,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// NSS AUTO/HARD
		{	'B',	9u	,	AA_GPIO_AF_0,	AA_GPIO_MODE_OUTPUT | AA_GPIO_SPEED_LOW    },		// NSS USER/SOFT
		{	// DMA Tx
			DMA_DMA1,							//	DMA number 1 or 2, 0 => no DMA
			3u,									//	DMA stream number 0..6
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction: LL_DMA_DIRECTION_MEMORY_TO_PERIPH
			(BSP_MAX_INT_PRIO+2u),				//	DMA Interrupt priority
		},
		{	// DMA Rx
			DMA_DMA1,							//	DMA number 1 or 2, 0 => no DMA
			4u,									//	DMA stream number 0..6
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY,	//	Direction: LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			(BSP_MAX_INT_PRIO+2u),				//	DMA interrupt priority
		},
	},
	#endif	// WITH_SPI_2
} ;

//------------------------------------------------------------------------------
#endif	// spiL4xx_H_
