/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	timG0xx.c		STM32G0xx Timer descriptors for timbasic.c

	When		Who	What
	06/02/21	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2021 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined timG0xx_H_
#define timG0xx_H_
//------------------------------------------------------------------------------
/*
TIM		AF	CH1		CH1N	CH2		CH2N	CH3		CH3N	CH4		BKIN	ETR
--------------------------------------------------------------------------------
TIM1	1/2	PA8		PB13	PA9		PB0		PA10	PB1		PA11	PC13	PA12
							PB3		PB14	PB6		PB15	PC11	PC14
			PC8		PD2		PC9		PD3		PC10	PD4				PA6
																	PB12
																	PD5

TIM2	2	PA0				PA1				PA2				PA3				PA0
			PA5				PB3				PC6				PC7				PA5
			PC4				PC5				PB10			PB11			PC4
			PA15															PA15

TIM3	1	PA6				PA7
			PB4				PB5				PB0				PB1
			PC6				PC7				PC8				PC9				PD2

TIM4	1	PB6				PB7				PB8				PB9
		2	PD12			PD13			PD14			PD15			PE0
TIM6		No I/O

TIM7		No I/O

TIM14	2	PC12
		2	PF0
		4	PA4
		4	PA7
		0	PB1

TIM15	2/4	PC1		PF1		PC2										PC15
		/5	PA2		PA1		PA3										PB12
			PB14	PB13											PB8
					PB15	PB15									PA9

TIM16	2/5	PA6		PB6												PB5
			PD0
			PB8

TIM17	2/5	PA7		PB7												PA10
			PD1														PB4
			PB9

*/

//------------------------------------------------------------------------------
// In this descriptor array example, at least one descriptors for each timer configuration

// Some timer manage more than 1 interrupt vector: TIM1 and TIM8
// The descriptor describe only 1 interrupt.
// If you need more than 1 interrupt vector, configure the other interrupts from your application code

const	timDesc_t	timDescArray [] =
{
	#if (WITH_TIM_1 == 1)
	{
		1,									// Descriptor identifier
		1,									// Timer number 1..n
//		RCC_APB2RSTR_TIM1RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB2ENR_TIM1EN_Pos,				// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_ADVT | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR | TIM_CAP_BREAK | TIM_CAP_COMP_OUT, 	// Capabilities flags
		TIM1,								// Pointer to TIM registers
		TIM1_BRK_UP_TRG_COM_IRQn,
		{
			{	'A',	8,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
			{	'A',	9,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch2
			{	'A',	10,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch3
			{	'A',	11,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch4
			{	'B',	13,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1N
			{	'B',	0,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch2N
			{	'B',	1,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch3N
			{	'A',	12,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// ETR
			{	'A',	6,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// BKIN
		},
	},
	#endif	// WITH_TIM1

	#if (WITH_TIM_2 == 1)
	{
		2,									// Descriptor identifier
		2,									// Timer number 1..n
//		RCC_APB1RSTR_TIM2RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM2EN_Pos,				// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR | TIM_CAP_32_BITS, 	// Capabilities flags
		TIM2,								// Pointer to TIM registers
		TIM2_IRQn,
		{
			{	'A',	0,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
			{	'A',	1,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch2
			{	'A',	2,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch3
			{	'A',	3,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch4
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch1N
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch2N
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch3N
			{	'A',	5,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// ETR
			{	0,		0,	AA_GPIO_AF_2,	0 },	// BKIN
		},
	},
	#endif	// WITH_TIM2

	#if (WITH_TIM_3 == 1)
	{
		3,									// Descriptor identifier
		3,									// Timer number 1..n
//		RCC_APB1RSTR_TIM3RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM3EN_Pos,				// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR, 	// Capabilities flags
		TIM3,								// Pointer to TIM registers
		#if (defined TIM4)
			TIM3_TIM4_IRQn,
		#else
			TIM3_IRQn,
		#endif
		{
			{	'C',	6,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
			{	'C',	7,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch2
			{	'C',	8,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch3
			{	'C',	9,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch4
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch1N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch2N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch3N
			{	'D',	2,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// ETR
			{	0,		0,	AA_GPIO_AF_1,	0 },	// BKIN
		},
	},
	#endif	// WITH_TIM3
	
#if (WITH_TIM_4 == 1)
{
	4,									// Descriptor identifier
	4,									// Timer number 1..n
//		RCC_APB1RSTR_TIM3RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM3EN_Pos,				// RCC_APBxRSTR_TIMnEN_Pos
	TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR, 	// Capabilities flags
	TIM4,								// Pointer to TIM registers
	TIM3_TIM4_IRQn,
	{
		{	'D',	12,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
		{	'D',	13,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch2
		{	'D',	14,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch3
		{	'D',	15,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch4
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch1N
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch2N
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch3N
		{	'E',	0,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// ETR
		{	0,		0,	AA_GPIO_AF_2,	0 },	// BKIN
	},
},
#endif	// WITH_TIM3

	#if (WITH_TIM_6 == 1)
	{
		6,									// Descriptor identifier
		6,									// Timer number 1..n
//		RCC_APB1RSTR_TIM6RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM6EN_Pos,				// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_BASIC | TIM_CAP_DMA, 	// Capabilities flags
		TIM6,								// Pointer to TIM registers
		#if (defined LPTIM1)
			TIM6_DAC_LPTIM1_IRQn,
		#else
			TIM6_IRQn,
		#endif
		{
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch1
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch2
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch3
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch4
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch1N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch2N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch3N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// ETR
			{	0,		0,	AA_GPIO_AF_1,	0 },	// BKIN
		},
	},
	#endif	// WITH_TIM6

	#if (WITH_TIM_7 == 1)
	{
		7,									// Descriptor identifier
		7,									// Timer number 1..n
//		RCC_APB1RSTR_TIM7RST_Pos,			/ RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM7EN_Pos,				// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_BASIC | TIM_CAP_DMA, 	// Capabilities flags
		TIM7,								// Pointer to TIM registers
		#if (defined LPTIM2)
			TIM7_LPTIM2_IRQn,
		#else
			TIM7_IRQn,
		#endif
		{
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch1
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch2
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch3
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch4
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch1N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch2N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// Ch3N
			{	0,		0,	AA_GPIO_AF_1,	0 },	// ETR
			{	0,		0,	AA_GPIO_AF_1,	0 },	// BKIN
		},
	},
	#endif	// WITH_TIM7

	#if (WITH_TIM_14 == 1)
	{
		14,									// Descriptor identifier
		14,									// Timer number 1..n
//		RCC_APB1RSTR_TIM14RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM14EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT2 | TIM_CAP_CHAN_1, // Capabilities flags
		TIM14,								// Pointer to TIM registers
		TIM14_IRQn,
		{
			{	'C',	12,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch2
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch3
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch4
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch1N
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch2N
			{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch3N
			{	0,		0,	AA_GPIO_AF_2,	0 },	// ETR
			{	0,		0,	AA_GPIO_AF_2,	0 },	// BKIN
		},
	},
	#endif	// WITH_TIM14

#if (WITH_TIM_15 == 1)
{
	15,									// Descriptor identifier
	15,									// Timer number 1..n
//		RCC_APB1RSTR_TIM15RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM15EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
	TIM_CAP_TYPE_GPT2 | TIM_CAP_CHAN_2 | TIM_CAP_BREAK, // Capabilities flags
	TIM15,								// Pointer to TIM registers
	TIM15_IRQn,
	{
		{	'C',	1,	AA_GPIO_AF_2,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch2
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch3
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch4
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch1N
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch2N
		{	0,		0,	AA_GPIO_AF_2,	0 },	// Ch3N
		{	0,		0,	AA_GPIO_AF_2,	0 },	// ETR
		{	0,		0,	AA_GPIO_AF_2,	0 },	// BKIN
	},
},
#endif	// WITH_TIM15

#if (WITH_TIM_16 == 1)
{
	16,									// Descriptor identifier
	16,									// Timer number 1..n
//		RCC_APB1RSTR_TIM16RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM16EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
	TIM_CAP_TYPE_GPT2 | TIM_CAP_CHAN_1 | TIM_CAP_BREAK, // Capabilities flags
	TIM16,								// Pointer to TIM registers
	TIM16_IRQn,
	{
		{	'A',	6,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch2
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch3
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch4
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch1N
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch2N
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch3N
		{	0,		0,	AA_GPIO_AF_5,	0 },	// ETR
		{	0,		0,	AA_GPIO_AF_5,	0 },	// BKIN
	},
},
#endif	// WITH_TIM16

#if (WITH_TIM_17 == 1)
{
	17,									// Descriptor identifier
	17,									// Timer number 1..n
//		RCC_APB1RSTR_TIM17RST_Pos,			// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM17EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
	TIM_CAP_TYPE_GPT2 | TIM_CAP_CHAN_1 | TIM_CAP_BREAK, // Capabilities flags
	TIM17,								// Pointer to TIM registers
	TIM17_IRQn,
	{
		{	'A',	7,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_MEDIUM | AA_GPIO_PUSH_PULL },	// Ch1
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch2
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch3
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch4
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch1N
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch2N
		{	0,		0,	AA_GPIO_AF_5,	0 },	// Ch3N
		{	0,		0,	AA_GPIO_AF_5,	0 },	// ETR
		{	0,		0,	AA_GPIO_AF_5,	0 },	// BKIN
	},
},
#endif	// WITH_TIM17


	// Used by some aaBasic tests
	#if (WITH_TIM_2 == 1)
	{
		20,								// Descriptor identifier
		2,								// Timer number 1..14
//		RCC_APB1RSTR_TIM2RST_Pos,		// RCC_APBxRSTR_TIMnRST_Pos
//		RCC_APB1ENR_TIM2EN_Pos,			// RCC_APBxRSTR_TIMnEN_Pos
		TIM_CAP_TYPE_GPT1 | TIM_CAP_CHAN_1_4 | TIM_CAP_UP_DOWN | TIM_CAP_DMA | TIM_CAP_ETR | TIM_CAP_32_BITS, 	// Capabilities flags
		TIM2,							// Pointer to TIM registers
		TIM2_IRQn,
		{
			{	'A',	0,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_HIGH | AA_GPIO_PUSH_PULL },		// Ch1
			{	0,		3,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_HIGH | AA_GPIO_PUSH_PULL },		// Ch2
			{	0,		10,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_HIGH | AA_GPIO_PUSH_PULL },		// Ch3
			{	0,		3,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_HIGH | AA_GPIO_PUSH_PULL },		// Ch4
			{	0,		0,	AA_GPIO_AF_1,	0 },		// Ch1N
			{	0,		0,	AA_GPIO_AF_1,	0 },		// Ch2N
			{	0,		0,	AA_GPIO_AF_1,	0 },		// Ch3N
			{	0,		15,	AA_GPIO_AF_1,	AA_GPIO_MODE_ALTERNATE | AA_GPIO_SPEED_HIGH | AA_GPIO_PUSH_PULL },		// ETR
			{	0,		0,	AA_GPIO_AF_1,	0 },		// BKIN
		},
	},
	#endif	// WITH_TIM2

} ;
//------------------------------------------------------------------------------
#endif	// timG0xx_H_
